@@ -16,6 +16,7 @@ import numpy as np
 
 from clasp import click
 import clasp.click_ext as clk
+from clasp.script_tools import try_mkdir
 
 import raytraverse
 from raytraverse import translate
@@ -26,7 +27,8 @@ from raytraverse.mapper import PlanMapper, SkyMapper, ViewMapper
 from raytraverse.utility.cli import np_load, np_load_safe, shared_pull, pull_decs
 from raytraverse.scene import Scene
 from raytraverse.renderer import Rtrace, Rcontrib
-from raytraverse.sampler import SkySamplerPt, SamplerArea, SamplerSuns
+from raytraverse.sampler import SkySamplerPt, SamplerArea, SamplerSuns, \
+    SrcSamplerPt
 from raytraverse.sky import SkyData, skycalc
 
 
@@ -105,11 +107,6 @@ engine_opts = [
                    ' variance to sample. A value of 1 will have a sample count'
                    ' at the final sampling level equal to the number of'
                    ' directions with a contribution variance greater than .25'),
- click.option("-vlt", default=0.64,
-              help="primary transmitting vlt, used to scale the accuracy "
-                   "parameter to the expected scene variance. Optional, but "
-                   "helpful with, for example, electrochromic glazing or "
-                   "shades"),
  click.option('-idres', default=32,
               help='the initial directional sampling resolution '
                    '(as sqrt of samples per hemisphere)'),
@@ -255,7 +252,8 @@ def suns(ctx, loc=None, opts=False, debug=False, version=False, epwloc=False,
         else:
             xyz = sm.uv2xyz(sm.solarbounds.vertices)
             xyz = xyz[np.logical_not(np.isnan(xyz[:, 0]))]
-        click.echo("columns: 0:x 1:y 2:z 3:x(anglular) 4:y(angular) 5:u 6:v", err=True)
+        click.echo("columns: 0:x 1:y 2:z 3:x(anglular) 4:y(angular) 5:u 6:v",
+                   err=True)
         uv = sm.xyz2uv(xyz)
         axy = translate.xyz2xy(xyz)
         for x, u, a in zip(xyz, uv, axy):
@@ -341,6 +339,11 @@ def skydata(ctx, wea=None, name="skydata", loc=None, reload=True, opts=False,
 @click.option('-dcompargs', default='-ab 1',
               help="additional arguments for running direct component. when "
                    "using, set -ab in sunengine.rayargs to this ab minus one.")
+@click.option("-vlt", default=0.64,
+              help="primary transmitting vlt, used to scale the accuracy "
+                   "parameter to the expected scene variance. Optional, but "
+                   "helpful with, for example, electrochromic glazing or "
+                   "shades")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def skyengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
               default_args=True, skyres=15, nlev=5, dcompargs='-ab 1',
@@ -372,6 +375,11 @@ def skyengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
 @click.option('-nlev', default=6,
               help='number of directional sampling levels, yielding a final'
                    'resolution of idres^2 * 2^(nlev) samples per hemisphere')
+@click.option("-vlt", default=0.64,
+              help="primary transmitting vlt, used to scale the accuracy "
+                   "parameter to the expected scene variance. Optional, but "
+                   "helpful with, for example, electrochromic glazing or "
+                   "shades")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def sunengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
               default_args=True, nlev=6, opts=False,
@@ -384,6 +392,9 @@ def sunengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
     """
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
+    if 'sourceengine' in ctx.obj:
+        ctx.obj['sourceengine']['engine'].reset()
+        ctx.obj.pop('sourceengine')
     scn = ctx.obj['scene']
     rtrace = Rtrace(rayargs=rayargs, scene=scn.scene, default_args=default_args)
     accuracy = accuracy*vlt/0.64
@@ -391,6 +402,50 @@ def sunengine(ctx, accuracy=1.0, vlt=0.64, idres=32, rayargs=None,
     ctx.obj['sunengine'] = dict(engine=rtrace, ptkwargs=ptkwargs)
 
 
+@main.command()
+@clk.shared_decs(engine_opts)
+@click.option("-srcfile", default=None, type=click.Path(dir_okay=False),
+              help="scene source description (required)")
+@click.option("-source", default="source", help="name for this source")
+@click.option('-nlev', default=6,
+              help='number of directional sampling levels, yielding a final'
+                   'resolution of idres^2 * 2^(nlev) samples per hemisphere')
+@click.option("-vlt", default=1.0,
+              help="Leave at 1.0 for interior light sources. primary "
+                   "transmitting vlt, used to scale the accuracy "
+                   "parameter to the expected scene variance. Optional, but "
+                   "helpful with, for example, electrochromic glazing or "
+                   "shades")
+@click.option("--color/--no-color", default=True)
+@clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
+def sourceengine(ctx, srcfile=None, source="source", accuracy=1.0, vlt=1.0,
+                 idres=32, rayargs=None, default_args=True, nlev=6, color=True,
+                 opts=False, debug=False, version=None, **kwargs):
+    """initialize engine for sunrun
+
+    Effects
+    ~~~~~~~
+        - Invokes scene
+    """
+    if srcfile is None:
+        click.echo("srcfile is required by sourcerun", err=True)
+        raise click.Abort
+    if 'scene' not in ctx.obj:
+        clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
+    if 'sunengine' in ctx.obj:
+        ctx.obj['sunengine']['engine'].reset()
+        ctx.obj.pop('sunengine')
+    scn = ctx.obj['scene']
+    srcscn = scn.source_scene(srcfile, source)
+    rtrace = Rtrace(rayargs=rayargs, scene=srcscn, default_args=default_args)
+    if color:
+        rtrace.update_ospec("v")
+    accuracy = accuracy*vlt/0.64
+    ptkwargs = dict(accuracy=accuracy, idres=idres, nlev=nlev, stype=source)
+    ctx.obj['sourceengine'] = dict(engine=rtrace, ptkwargs=ptkwargs,
+                                   source=(srcfile, source))
+
+
 sample_opts = [
  click.option("-accuracy", default=1.0,
               help="parameter to set threshold at sampling level relative to "
@@ -453,8 +508,12 @@ def skyrun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False, plotp=False,
         skyfield = LightPlaneKD(scn, f"{scn.outdir}/{pm.name}/sky_points"
                                 f".tsv", pm, "sky")
     except OSError:
-        skyfield = skysampler.run(pm, plotp=plotp, pfish=False,
-                                  find_reflections=True)
+        skyfield = skysampler.run(pm, plotp=plotp, pfish=False)
+        try_mkdir(f"{scn.outdir}/{pm.name}")
+        reflf = f"{scn.outdir}/{pm.name}/reflection_normals.txt"
+        refl = scn.reflection_search(skyfield.vecs)
+        if refl.size > 0:
+            np.savetxt(reflf, refl)
     else:
         if scn.dolog:
             click.echo(f"Sky Lightfield reloaded from {scn.outdir}/{pm.name} "
@@ -549,6 +608,12 @@ def sunrun(ctx, srcaccuracy=1.0, srcnlev=3, srcjitter=True, recover=False,
             shutil.rmtree(src, ignore_errors=True)
         for src in glob(f"{scn.outdir}/{pm.name}/i_{skmapper.name}_sun_*"):
             shutil.rmtree(src, ignore_errors=True)
+    try_mkdir(f"{scn.outdir}/{pm.name}")
+    reflf = f"{scn.outdir}/{pm.name}/reflection_normals.txt"
+    if not os.path.isfile(reflf):
+        refl = scn.reflection_search(pm.point_grid(False))
+        if refl.size > 0:
+            np.savetxt(reflf, refl)
     dfield = sunsampler.run(skmapper, pm, specguide=True, recover=recover,
                             plotp=plotp, pfish=False)
     ctx.obj['daylightfield'] = dfield
@@ -556,16 +621,23 @@ def sunrun(ctx, srcaccuracy=1.0, srcnlev=3, srcjitter=True, recover=False,
 
 @main.command()
 @clk.shared_decs(sample_opts)
-@click.option("-srcfile", default=None, type=click.Path(dir_okay=False),
-              help="scene source description (required)")
-@click.option("-source", default="source", help="name for this source")
 @click.option("--overwrite/--no-overwrite", default=False,
               help="If True, reruns sampler when invoked, otherwise will first"
                    " attempt to load results")
+@click.option("--scenedetail/--no-scenedetail", default=False,
+              help="If True, includes scene details (distance, surface normal,"
+                   "and modifier as features). Increases sampling rate to"
+                   " improve image reconstruction")
+@click.option("-distance", default=0.5,
+              help="when using scene detail, the difference in ray length "
+                   "equivalent to final sampling luminance threshold")
+@click.option("-normal", default=5.0,
+              help="when using scene detail, the difference in surface normal "
+                   "(degrees) equivalent to final sampling luminance threshold")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def sourcerun(ctx, srcfile=None, source="source", accuracy=1.0, nlev=3,
-              jitter=True, overwrite=False, plotp=False, edgemode='constant',
-              opts=False, debug=False, version=None):
+def sourcerun(ctx, accuracy=1.0, nlev=3, jitter=True, overwrite=False,
+              plotp=False, edgemode='constant', scenedetail=False, distance=0.5,
+              normal=5.0, opts=False, debug=False, version=None):
     """run scene for a single source (or multiple defined in a single scene file)
 
     - Do not run as part of the same call as sunrun
@@ -581,23 +653,22 @@ def sourcerun(ctx, srcfile=None, source="source", accuracy=1.0, nlev=3,
         - creates outdir/area.name/sky/SOURCE/######.rytpt
             - each file is a LightPointKD initialization object
     """
-    if srcfile is None:
-        click.echo("srcfile is required by sourcerun", err=True)
-        raise click.Abort
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
     scn = ctx.obj['scene']
     if 'planmapper' not in ctx.obj:
         clk.invoke_dependency(ctx, area)
     pm = ctx.obj['planmapper']
-    if 'sunengine' not in ctx.obj:
-        clk.invoke_dependency(ctx, sunengine)
-    snengine = ctx.obj['sunengine']['engine']
-    ptkwargs = ctx.obj['sunengine']['ptkwargs']
+    if 'sourceengine' not in ctx.obj:
+        clk.invoke_dependency(ctx, sourceengine)
+    srcrtrace = ctx.obj['sourceengine']['engine']
+    ptkwargs = ctx.obj['sourceengine']['ptkwargs']
+    srcfile, source = ctx.obj['sourceengine']['source']
     if ctx.obj['static_points']:
         nlev = 1
         jitter = False
-    srcengine = SkySamplerPt(scn, snengine, **ptkwargs)
+    srcengine = SrcSamplerPt(scn, srcrtrace, srcfile, scenedetail=scenedetail,
+                             distance=distance, normal=normal, **ptkwargs)
     srcsampler = SamplerArea(scn, srcengine, accuracy=accuracy, nlev=nlev,
                              jitter=jitter, edgemode=edgemode)
     try:
@@ -606,8 +677,14 @@ def sourcerun(ctx, srcfile=None, source="source", accuracy=1.0, nlev=3,
         skyfield = LightPlaneKD(scn, f"{scn.outdir}/{pm.name}/{source}_points"
                                 f".tsv", pm, source)
     except OSError:
+        try_mkdir(f"{scn.outdir}/{pm.name}")
+        reflf = f"{scn.outdir}/{pm.name}/reflection_normals.txt"
+        if not os.path.isfile(reflf):
+            refl = scn.reflection_search(pm.point_grid(False))
+            if refl.size > 0:
+                np.savetxt(reflf, refl)
         skyfield = srcsampler.run(pm, plotp=plotp, pfish=False,
-                                  find_reflections=True)
+                                  specguide=True)
     else:
         if scn.dolog:
             click.echo(f"Source Lightfield reloaded from {scn.outdir}/{pm.name} "
@@ -645,8 +722,9 @@ rows in wea/epw file using space seperated list or python range notation:
  click.option("--maskfull/--maskday", default=True,
               help="if false, skymask assumes daystep indices"),
  click.option("-simtype", default="3comp",
-              type=click.Choice(api.stypes),
-              help=f"simulation process/integration type:\n\n{api.stypedocstring}"),
+              help=f"simulation process/integration type:\n\n"
+                   f"{api.stypedocstring}\n\nor source name (overrides "
+                   f"--rresampleview, --directview, etc."),
  click.option("-resuntol", default=1.0,
               help="tolerance for resampling sun views"),
  click.option("-resamprad", default=0.0,
@@ -661,7 +739,9 @@ rows in wea/epw file using space seperated list or python range notation:
 @main.command()
 @clk.shared_decs(eval_opts)
 @click.option("-res", default=800, help="image resolution")
-@click.option("-interpolate", type=click.Choice(['linear', 'fast', 'high', '', 'None', 'False']))
+@click.option("-interpolate", type=click.Choice(['linear', 'fast', 'high',
+                                                 'fastc', 'highc', '', 'None',
+                                                 'False']))
 @click.option("--namebyindex/--no-namebyindex", default=False,
               help="if False (default), names images by: "
                    "<prefix>_sky-<row>_pt-<x>_<y>_<z>_vd-<dx>_<dy>_<dz>.hdr "
@@ -695,25 +775,39 @@ def images(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene, reload=True, overwrite=False)
     scn = ctx.obj['scene']
-    if 'skydata' not in ctx.obj:
-        clk.invoke_dependency(ctx, skydata, reload=True)
-    sd = ctx.obj['skydata']
-    if directview:
-        sd = SkyData(None, skyres=sd.skyres)
     if 'planmapper' not in ctx.obj:
         clk.invoke_dependency(ctx, area)
     pm = ctx.obj['planmapper']
-    if 'skymapper' not in ctx.obj:
-        clk.invoke_dependency(ctx, suns)
-    skmapper = ctx.obj['skymapper']
-    if resampleview or interpolate in ['fast', 'high']:
-        if 'sunengine' not in ctx.obj:
-            clk.invoke_dependency(ctx, sunengine)
-        sunviewengine = ctx.obj['sunengine']['engine']
+    if simtype not in api.stypes:
+        sd = SkyData(None, skyres=1, ground=False, srcname=simtype)
+        resampleview = False
+        skymask = None
+        srcname = simtype
+        if interpolate in ['fastc', 'highc']:
+            if 'sourceengine' not in ctx.obj:
+                clk.invoke_dependency(ctx, sourceengine)
+            sunviewengine = ctx.obj['sourceengine']['engine']
+        else:
+            sunviewengine = None
     else:
-        sunviewengine = None
-
-    itg = api.get_integrator(scn, pm, skmapper.name, simtype,
+        if 'skydata' not in ctx.obj:
+            clk.invoke_dependency(ctx, skydata, reload=True)
+        sd = ctx.obj['skydata']
+        if 'skymapper' not in ctx.obj:
+            clk.invoke_dependency(ctx, suns)
+        skmapper = ctx.obj['skymapper']
+        if resampleview or interpolate in ['fastc', 'highc']:
+            if 'sunengine' not in ctx.obj:
+                clk.invoke_dependency(ctx, sunengine)
+            sunviewengine = ctx.obj['sunengine']['engine']
+        else:
+            sunviewengine = None
+        if directview:
+            sd = SkyData(None, skyres=sd.skyres)
+        srcname = skmapper.name
+    if not resampleview:
+        resuntol = 180
+    itg = api.get_integrator(scn, pm, srcname, simtype,
                              sunviewengine=sunviewengine)
     if skymask is not None:
         if maskfull:
@@ -724,7 +818,7 @@ def images(ctx, sensors=None, sdirs=None, viewangle=180., skymask=None,
 
     if interpolate == "linear":
         interpolate = True
-    elif interpolate not in ['fast', 'high']:
+    elif interpolate not in ['fastc', 'highc', 'fast', 'high']:
         interpolate = False
 
     if sensors.shape[1] == 6:
