@@ -8,12 +8,7 @@
 
 """sampler objects"""
 
-__all__ = ["Integrator", "IntegratorDS", "ZonalIntegrator", "ZonalIntegratorDS",
-           "IntegratorDV", "ZonalIntegratorDV"]
+__all__ = ["Integrator", "SensorIntegrator"]
 
 from raytraverse.integrator.integrator import Integrator
-from raytraverse.integrator.integratords import IntegratorDS
-from raytraverse.integrator.zonalintegrator import ZonalIntegrator
-from raytraverse.integrator.zonalintegratords import ZonalIntegratorDS
-from raytraverse.integrator.integratordv import IntegratorDV
-from raytraverse.integrator.zonalintegratordv import ZonalIntegratorDV
+from raytraverse.integrator.sensorintegrator import SensorIntegrator
