@@ -9,10 +9,13 @@
 """sampler objects"""
 
 __all__ = ["LightPlaneKD", "SunsPlaneKD", "LightField",
-           "LightResult", "ResultAxis", "RaggedResult", "ZonalLightResult"]
+           "LightResult", "ResultAxis", "RaggedResult", "ZonalLightResult",
+           "SensorPlaneKD", "SunSensorPlaneKD"]
 
 from raytraverse.lightfield.lightfield import LightField
 from raytraverse.lightfield.lightplanekd import LightPlaneKD
 from raytraverse.lightfield.sunsplanekd import SunsPlaneKD
+from raytraverse.lightfield.sensorplanekd import SensorPlaneKD
+from raytraverse.lightfield.sunsensorplanekd import SunSensorPlaneKD
 from raytraverse.lightfield.lightresult import ResultAxis, LightResult
 from raytraverse.lightfield.zonallightresult import RaggedResult, ZonalLightResult
