@@ -255,7 +255,7 @@ class LightResult(object):
             tl = False
         for rh in labels:
             if tl:
-                rl = "_".join(f"{i:.02f}" for i in rh)
+                rl = "_".join(f"{i:05.02f}" for i in rh)
             elif np.issubdtype(type(rh), np.integer):
                 rl = f"{rh:04d}"
             else:
@@ -330,7 +330,7 @@ class LightResult(object):
             data = rt[..., i]
             for k, (la, d) in enumerate(zip(flabels1, data.T)):
                 img[mask] = d[idx]
-                io.array2hdr(img, f"{basename}_{la}_{la0}.hdr", header)
+                io.array2hdr(img, f"{basename}_{la}_{la0}.hdr", [header])
 
     @staticmethod
     def _pull2hdr_sky(skyfill, basename, spd, rt, flabels0, flabels1):
