@@ -13,7 +13,7 @@ from clasp import click
 import clasp.click_ext as clk
 
 import raytraverse
-from raytraverse import translate
+from raytraverse import translate, api
 from raytraverse.lightfield import ResultAxis
 from raytraverse.lightfield import LightResult
 from raytraverse.sky import SkyData, skycalc
@@ -193,26 +193,60 @@ def imgmetric(ctx, imgs=None, metrics=None, parallel=True,
 @main.command()
 @click.option('-out', default="imglf", type=click.Path(file_okay=False),
               help="output directory")
-@click.option("-imga", callback=clk.are_files,
-              help="hdr image files, primary view direction, must be angular "
-                   "fisheye projection, view header required")
-@click.option("-imgb", callback=clk.are_files,
-              help="hdr image files, opposite view direction, must be angular "
-                   "fisheye projection, assumed to be same as imga with -vd"
-                   " reversed")
+@click.option("-img", callback=clk.are_files,
+              help="hdr image files, uv anr angular")
+@click.option("--uv2ang/--ang2uv", default=False,
+              help="direction of transform")
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def img2lf(ctx, imga=None, imgb=None, out="imglf", **kwargs):
-    """read and compress angular fisheye images into lightpoints/lightplane"""
-    if imga is None:
-        click.echo("-imga is required", err=True)
-        raise click.Abort
-    if imgb is None:
-        imgb = [None] * len(imga)
-    elif len(imgb) < len(imga):
-        imgb = imgb + [None] * (len(imga) - len(imgb))
-    scene = ImageScene(out)
-    srcs = [f"img{i:03d}" for i in range(len(imga))]
-    results = pool_call(imagetools.img2lf, list(zip(imga, imgb, srcs)), scn=scene)
+def project(ctx, img=None, uv2ang=False, **kwargs):
+    """project images between angular and shirley-chiu square coordinates"""
+    if uv2ang:
+        func = imagetools.hdr_uv2ang
+    else:
+        func = imagetools.hdr_ang2uv
+    results = pool_call(func, img, expandarg=False)
+
+
+# @main.command()
+# @click.option('-out', default="imglf", type=click.Path(file_okay=False),
+#               help="output directory")
+# @click.option("-imga", callback=clk.are_files,
+#               help="hdr image files, primary view direction, must be angular "
+#                    "fisheye projection, view header required")
+# @click.option("-imgb", callback=clk.are_files,
+#               help="hdr image files, opposite view direction, must be angular "
+#                    "fisheye projection, assumed to be same as imga with -vd"
+#                    " reversed")
+# @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
+# def img2lf(ctx, imga=None, imgb=None, out="imglf", **kwargs):
+#     """read and compress angular fisheye images into lightpoints/lightplane"""
+#     if imga is None:
+#         click.echo("-imga is required", err=True)
+#         raise click.Abort
+#     if imgb is None:
+#         imgb = [None] * len(imga)
+#     elif len(imgb) < len(imga):
+#         imgb = imgb + [None] * (len(imga) - len(imgb))
+#     scene = ImageScene(out)
+#     srcs = [f"img{i:03d}" for i in range(len(imga))]
+#     results = pool_call(imagetools.img2lf, list(zip(imga, imgb, srcs)), scn=scene)
+
+
+def _dview(x):
+    lpt = api.load_lp(x)
+    lpt.direct_view(res=800)
+    return lpt.lum.shape[0]
+
+@main.command()
+@click.option('-lp', callback=clk.are_files, required=True,
+              help="path to lightpoint(s)")
+@clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
+def lp2img(ctx, lp=None, **kwargs):
+    """make hdr directview of lightpoint"""
+    rays = pool_call(_dview, lp, expandarg=False)
+    click.echo("lightpoint \t samples")
+    for r, l in zip(rays, lp):
+        click.echo(f"{l} \t {r}")
 
 @main.command()
 @clk.shared_decs(pull_decs)
