@@ -5,9 +5,14 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
+import os
 import re
 import sys
+import tempfile
 
+import numpy as np
+
+from raytraverse import translate
 from raytraverse.renderer.radiancerenderer import RadianceRenderer
 from craytraverse.crenderer import cRtrace
 
@@ -59,13 +64,13 @@ class Rtrace(RadianceRenderer):
     successive calls to the instance.
     """
     name = 'rtrace'
+    #: craytraverse.crenderer.cRtrace
     instance = rtrace_instance
     defaultargs = (f"-u+ -ab 16 -av 0 0 0 -aa 0 -as 0 -dc 1 -dt 0 -lr -14 -ad "
                    f"1000 -lw 0.00004 -st 0 -ss 16 -w-")
     directargs = "-w- -av 0 0 0 -ab 0 -lr 1 -n 1"
     usedirect = False
     ospec = "Z"
-    ocnt = 1
 
     def __init__(self, rayargs=None, scene=None, nproc=None,
                  default_args=True, direct=False):
@@ -103,6 +108,8 @@ class Rtrace(RadianceRenderer):
         ospec = re.findall(r"-o\w+", cls.args)
         if len(ospec) > 0:
             cls.update_ospec(ospec[-1][2:])
+        else:
+            cls.update_ospec("Z")
 
     @classmethod
     def update_ospec(cls, vs):
@@ -144,7 +151,7 @@ class Rtrace(RadianceRenderer):
             raise ValueError(f"Could not update {cls.__name__} with "
                              f"outputs: '{vs}'")
         cls.ospec = vs
-        cls.ocnt = outcnt
+        cls.features = outcnt
         return outcnt
 
     @classmethod
@@ -188,7 +195,32 @@ class Rtrace(RadianceRenderer):
                       "from args", file=sys.stderr)
             cls.set_args(args)
 
+    @classmethod
+    def load_solar_source(cls, scene, sun, ambfile=None, intens=1):
+        # load new source
+        fd, srcdef = tempfile.mkstemp(dir=f"./{scene.outdir}/",
+                                      prefix='tmp_src')
+        try:
+            with os.fdopen(fd, 'w') as f:
+                f.write(scene.formatter.get_sundef(sun, (intens, intens,
+                                                         intens)))
+            cls.load_source(srcdef, ambfile=ambfile)
+        finally:
+            os.remove(srcdef)
+
     @classmethod
     def get_sources(cls):
-        """x,y,z,radius,area (x,y,z iis direction for distant sources)"""
-        return cls.instance.get_sources().reshape(-1, 5)
+        """returns source information
+
+        Returns
+        -------
+        sources: np.array
+            x,y,z,v,a
+            distant: direction, view angle, solid angle
+            not distant: location, max radius, area
+        distant: np.arrary
+            booleans, true if source type is distant
+        """
+        srcs, distant = cls.instance.get_sources()
+        srcs[distant, 3] = translate.chord2theta(srcs[distant, 3]) * 360/np.pi
+        return srcs, distant
