@@ -8,18 +8,22 @@
 
 """sampler objects"""
 
-__all__ = ['BaseSampler', 'SamplerArea', 'SamplerSuns', 'SamplerPt',
+__all__ = ['BaseSampler', 'SamplerArea', 'SamplerSuns', 'SamplerPt', 'Sensor',
            'SkySamplerPt', 'SunSamplerPtView', 'SunSamplerPt', 'ImageSampler',
-           'DeterministicImageSampler', 'SrcSamplerPt']
+           'DeterministicImageSampler', 'SrcSamplerPt', 'SrcSamplerPtView',
+           'ISamplerArea', 'ISamplerSuns']
 
 from raytraverse.sampler.basesampler import BaseSampler
 from raytraverse.sampler.samplerarea import SamplerArea
 from raytraverse.sampler.samplersuns import SamplerSuns
 from raytraverse.sampler.samplerpt import SamplerPt
 from raytraverse.sampler.sunsamplerptview import SunSamplerPtView
+from raytraverse.sampler.srcsamplerptview import SrcSamplerPtView
 from raytraverse.sampler.sunsamplerpt import SunSamplerPt
 from raytraverse.sampler.srcsamplerpt import SrcSamplerPt
 from raytraverse.sampler.skysamplerpt import SkySamplerPt
 from raytraverse.sampler.imagesampler import ImageSampler
 from raytraverse.sampler.imagesampler import DeterministicImageSampler
-
+from raytraverse.sampler.sensor import Sensor
+from raytraverse.sampler.isamplerarea import ISamplerArea
+from raytraverse.sampler.isamplersuns import ISamplerSuns
